SystemOrganization addCategory: #'XPath-Tests'!

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:18'!
testExpandedName
	self assert: (self nodeClass target: 'target' data: 'data') expandedName = 'target'! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:57'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:15'!
testHasNamespaceURI
	self deny: (self nodeClass target: 'target' data: 'data') hasNamespaceURI! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:16'!
testHasPrefix
	self deny: (self nodeClass target: 'target' data: 'data') hasPrefix! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:37'!
testHasPrefixedNamespaceURI
	| pi |

	pi := self nodeClass target: 'target' data: 'data'.
	self
		deny: (pi hasPrefixedNamespaceURI: 'target');
		deny: (pi hasPrefixedNamespaceURI: 'data');
		deny: (pi hasPrefixedNamespaceURI: '').! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:14'!
testLocalName
	self assert: (self nodeClass target: 'target' data: 'data') localName = 'target'! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:14'!
testName
	self assert: (self nodeClass target: 'target' data: 'data') name = 'target'! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:15'!
testNamespaceURI
	self assert: (self nodeClass target: 'target' data: 'data') namespaceURI isEmpty! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 14:07'!
testPrefix
	self assert: (self nodeClass target: 'target' data: 'data') prefix isEmpty! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass
			target: 'target'
			data: 'data') stringValue = 'data'.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:57'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:40'!
testHasNamespaceNodes
	| element |

	element := self nodeClass new.
	self assert: element hasNamespaceNodes.

	element declareNamespaces: self namespaces.
	self assert: element hasNamespaceNodes.

	element configuration usesNamespaces: false.
	self deny: element hasNamespaceNodes.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:35'!
testHasPrefixedNamespaceURI
	| element |

	element := self nodeClass name: 'test' namespaces: self namespaces.
	self
		assert: element namespaceURI notEmpty;
		assert: element prefix isEmpty;
		deny: (element hasPrefixedNamespaceURI: '').

	element name: 'prefix:test'.
	self assert: (element hasPrefixedNamespaceURI: 'prefixNS').! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:01'!
testIsPrincipalNode
	self assert: self nodeClass new isPrincipalNode! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 23:39'!
testNamespaceNodes
	| element nodes |

	element := self nodeClass new.
	nodes :=
		XMLNodeList with:
			(XPathNamespaceNode withXMLNamespaceOnElement: element).
	self assert: element namespaceNodes = nodes.

	element configuration usesNamespaces: false.
	self assert: element namespaceNodes isEmpty.

	element configuration usesNamespaces: true.
	element declareNamespaces: self namespaces.
	self namespaces keysAndValuesDo: [:prefix :uri |
		nodes addLast:
			(XPathNamespaceNode
				element: element
				name: prefix
				uri: uri)].
	self assert: element namespaceNodes = nodes.

	element declarePrefix: 'xml' uri: XMLNamespaceScope xmlNamespaceURI.
	nodes addLast: nodes removeFirst.
	self assert: element namespaceNodes = nodes.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 23:39'!
testNamespaceNodesDo
	| element |

	element := self nodeClass new.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: element namespaceNodes.

	element configuration usesNamespaces: false.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: #().

	element configuration usesNamespaces: true.
	element declareNamespaces: self namespaces.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: element namespaceNodes.

	element declarePrefix: 'xml' uri: XMLNamespaceScope xmlNamespaceURI.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: element namespaceNodes.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass new
			addContent: 'before';
			addNode:
				((self nodeClass named: 'element')
					addContent: 'content';
					yourself);
			addContent: 'after';
			yourself) stringValue = 'beforecontentafter'.! !

XMLNodeListTest subclass: #XPathNodeSetTest
	instanceVariableNames: 'document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathNodeSetTest methodsFor: 'testing' stamp: 'monty 1/18/2015 11:26'!
allowsDuplicates
	^ false! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 3/25/2015 23:55'!
assertHasUnknownSortOrder: aList
	self
		assertList: aList
		hasSortOrder: XPathUnknownSortOrder! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 09:34'!
assertList: aList copiedFrom: aSecondList equals: aThirdList
	super
		assertList: aList
		copiedFrom: aSecondList
		equals: aThirdList.
	self
		assert: aList set size = aList size;
		assert: aList set = aThirdList set;
		deny: aList set == aSecondList set.! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 3/25/2015 22:50'!
assertList: aFirstList hasSameSortOrderAs: aSecondList
	self assert: aFirstList sortOrder class == aSecondList sortOrder class! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 3/25/2015 23:56'!
assertList: aList hasSortOrder: aSortOrder
	self assert: (aList sortOrder isKindOf: aSortOrder)! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 09:35'!
assertList: aList isCopyOf: aSecondList
	super assertList: aList isCopyOf: aSecondList.
	self
		assert: aList set size = aList size;
		assert: aList set = aSecondList set;
		deny: aList set == aSecondList set.! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 1/18/2015 09:11'!
listClass
	^ XPathNodeSet! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 3/25/2015 22:15'!
listElements
	| listElements |

	listElements := super listElements.
	document
		ifNil: [
			(document := XMLDocument new)
				addElementNamed: 'root'.
			document root addNodes: listElements].
	^ listElements.
	! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 3/25/2015 22:42'!
listWithSortOrder
	^ self listWithElements sortOrder: XPathDocumentSortOrder new! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 1/31/2015 03:02'!
reverseSortBlock
	^ [:a :b | b occursInDocumentBeforeNode: a]! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 1/31/2015 03:02'!
sortBlock
	^ [:a :b | a = b or: [a occursInDocumentBeforeNode: b]]! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:43'!
testAdd
	super testAdd.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:45'!
testAddAfter
	super testAddAfter.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement after: self singleElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:47'!
testAddAfterIndex
	super testAddAfterIndex.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement afterIndex: 1;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:44'!
testAddAll
	super testAddAll.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			addAll: (Array with: self newElement);
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:44'!
testAddAllFirst
	super testAddAllFirst.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			addAllFirst: (Array with: self newElement);
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:44'!
testAddAllLast
	super testAddAllLast.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			addAllLast: (Array with: self newElement);
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:46'!
testAddBefore
	super testAddBefore.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement before: self singleElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:46'!
testAddBeforeIndex
	super testAddBeforeIndex.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement beforeIndex: 1;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:43'!
testAddFirst
	super testAddFirst.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			addFirst: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:43'!
testAddLast
	super testAddLast.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			addLast: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:00'!
testAllButFirst
	| list |

	super testAllButFirst.

	list := self listWithSortOrder.
	self
		assertList: list allButFirst
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:01'!
testAllButFirstFew
	| list |

	super testAllButFirstFew.

	list := self listWithSortOrder.
	self
		assertList: (list allButFirst: 1)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:00'!
testAllButLast
	| list |

	super testAllButLast.

	list := self listWithSortOrder.
	self
		assertList: list allButLast
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:02'!
testAllButLastFew
	| list |

	super testAllButLastFew.

	list := self listWithSortOrder.
	self
		assertList: (list allButLast: 1)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:42'!
testAsXPathBoolean
	self
		deny: self emptyList asXPathBoolean;
		assert: self listWithSingleElement asXPathBoolean! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:42'!
testAsXPathString
	self
		assert: self emptyList asXPathString isEmpty;
		assert: self listWithElements asXPathString = self singleElement stringValue;
		assert: self listWithElements reversed asXPathString = self singleElement stringValue! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:47'!
testAtPut
	super testAtPut.

	self assertHasUnknownSortOrder:
		(self listWithSortOrder
			at: 1 put: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:49'!
testCopy
	| list |

	super testCopy.

	list := self listWithSortOrder.
	self assertList: list copy hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:53'!
testCopyAfter
	| list |

	super testCopyAfter.

	list := self listWithSortOrder.
	self
		assertList: (list copyAfter: list first)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:54'!
testCopyAfterLast
	| list |

	super testCopyAfterLast.

	list := self listWithSortOrder.
	self
		assertList: (list copyAfterLast: list first)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:32'!
testCopyEmpty
	| list |

	super testCopyEmpty.

	list := self listWithSortOrder.
	self
		assertList: list copyEmpty
		hasSameSortOrderAs: self emptyList.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:52'!
testCopyFromTo
	| list |

	super testCopyFromTo.

	list := self listWithSortOrder.
	self
		assertList: (list copyFrom: 1 to: list size)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:59'!
testCopyReplaceAllWith
	| list |

	super testCopyReplaceAllWith.

	list := self listWithSortOrder.
	self assertHasUnknownSortOrder:
		(list
			copyReplaceAll: self listElements
			with: self listElements).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:59'!
testCopyReplaceFromToWith
	| list |

	super testCopyReplaceFromToWith.

	list := self listWithSortOrder.
	self assertHasUnknownSortOrder:
		(list
			copyReplaceFrom: 1
			to: list size
			with: self listElements).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:52'!
testCopyUpTo
	| list |

	super testCopyUpTo.

	list := self listWithSortOrder.
	self
		assertList: (list copyUpTo: list last)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:52'!
testCopyUpToLast
	| list |

	super testCopyUpToLast.

	list := self listWithSortOrder.
	self
		assertList: (list copyUpToLast: list last)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:55'!
testCopyWith
	| list |

	super testCopyWith.

	list := self listWithSortOrder.
	self assertHasUnknownSortOrder: (list copyWith: self newElement).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:56'!
testCopyWithFirst
	| list |

	super testCopyWithFirst.

	list := self listWithSortOrder.
	self assertHasUnknownSortOrder: (list copyWithFirst: self newElement).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:50'!
testCopyWithout
	| list |

	super testCopyWithout.

	list := self listWithSortOrder.
	self
		assertList: (list copyWithout: self singleElement)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:50'!
testCopyWithoutAll
	| list |

	super testCopyWithoutAll.

	list := self listWithSortOrder.
	self
		assertList: (list copyWithoutAll: (Array with: self singleElement))
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:33'!
testDefaultSortOrder
	self
		assertList: self emptyList hasSortOrder: XPathLevelDocumentSortOrder;
		assertList: self listWithSingleElement hasSortOrder: XPathLevelDocumentSortOrder;
		assertHasUnknownSortOrder: self listWithElements! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:17'!
testFirstFew
	| list |

	super testFirstFew.

	list := self listWithSortOrder.
	self
		assertList: (list first: list size)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:28'!
testFirstInDocumentOrder
	self
		assert: self emptyList firstInDocumentOrder isNil;
		assert: self listWithSingleElement firstInDocumentOrder = self singleElement;
		assert: self listWithElements firstInDocumentOrder = self listElements first;
		assert: self listWithElements reversed firstInDocumentOrder = self listElements first! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:17'!
testLastFew
	| list |

	super testLastFew.

	list := self listWithSortOrder.
	self
		assertList: (list last: list size)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:36'!
testReject
	| list |

	super testReject.

	list := self listWithSortOrder.
	self
		assertList: (list reject: [:each | false])
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:29'!
testRemoveAll
	| list |

	super testRemoveAll.

	(list := self listWithSortOrder) removeAll.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:29'!
testRemoveFirst
	| list |

	super testRemoveFirst.

	(list := self listWithSortOrder) removeFirst.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:28'!
testRemoveFirstFew
	| list |

	super testRemoveFirstFew.

	(list := self listWithSortOrder) removeFirst: 1.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:59'!
testRemoveIfAbsent
	| list |

	super testRemoveIfAbsent.

	(list := self listWithSortOrder)
		remove: self absentElement ifAbsent: [nil];
		remove: self singleElement ifAbsent: [nil].
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:28'!
testRemoveLast
	| list |

	super testRemoveLast.

	(list := self listWithSortOrder) removeLast.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:28'!
testRemoveLastFew
	| list |

	super testRemoveLastFew.

	(list := self listWithSortOrder) removeLast: 1.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:24'!
testReplaceFromToWith
	| list |

	super testReplaceFromToWith.

	(list := self listWithSortOrder)
		replaceFrom: 1
		to: list size
		with: self listElements.
	self assertHasUnknownSortOrder: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:25'!
testReplaceFromToWithStartingAt
	| list |

	super testReplaceFromToWithStartingAt.

	(list := self listWithSortOrder)
		replaceFrom: 1
		to: list size
		with: self listElements
		startingAt: 1.
	self assertHasUnknownSortOrder: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:33'!
testReverseSort
	| list |

	list := self listWithElements.
	self
		assert: list reverseSort == list;
		assert: list = self listWithElements reversed;
		assertList: list hasSortOrder: XPathReverseDocumentSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:34'!
testReverseSorted
	| list reverseSorted |

	list := self listWithElements.
	reverseSorted := self listWithElements reverseSorted.
	self
		assertList: reverseSorted
		copiedFrom: list
		equals: self listWithElements reversed.
	self assertList: reverseSorted hasSortOrder: XPathReverseDocumentSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:36'!
testSelect
	| list |

	super testSelect.

	list := self listWithSortOrder.
	self
		assertList: (list select: [:each | true])
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/26/2015 00:00'!
testSort
	super testSort.

	self
		assertList: self listWithElements sort
		hasSortOrder: XPathDocumentSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/26/2015 00:00'!
testSorted
	super testSorted.

	self
		assertList: self listWithElements sorted
		hasSortOrder: XPathDocumentSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:25'!
testSwapWith
	| list |

	super testSwapWith.

	(list := self listWithSortOrder)
		swap: 1
		with: list size.
	self assertHasUnknownSortOrder: list.! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 22:15'!
testDefaultContextPosition
	| element |

	super testDefaultContextPosition.

	element := self newElementWithAttributes.
	element attributeNodes withIndexDo: [:each :i |
		self assert: each defaultContextPosition = i].! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 22:16'!
testDefaultContextSize
	| element |

	super testDefaultContextSize.

	element := self newElementWithAttributes.
	element attributeNodesDo: [:each |
		self assert: each defaultContextSize = element attributeNodes size].! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:57'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:30'!
testHasPrefixedNamespaceURI
	| element namespaces |

	element := self newElementWithAttributes.
	namespaces := element namespaceScope prefixMappings values.
	element attributeNodesDo: [:each |
		each hasNamespaceURI
			ifTrue: [
				self assert:
					(each hasPrefixedNamespaceURI: each namespaceURI)]
			ifFalse: [
				self deny:
					(namespaces anySatisfy: [:namespace |
						each hasPrefixedNamespaceURI: namespace])]].! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:00'!
testIsPrincipalNode
	self assert: self nodeClass new isPrincipalNode! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass
			name: 'name'
			value: 'value') stringValue = 'value'! !

TestCase subclass: #XPathFunctionSetTest
	instanceVariableNames: 'functions disjointFunctions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:26'!
absentFunction
	^ XPathStringFunction! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/19/2015 16:22'!
assertSet: aFirstSet equals: aSecondSet
	self
		assert: aFirstSet = aSecondSet;
		assert: aSecondSet = aFirstSet! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/19/2015 16:23'!
denySet: aFirstSet equals: aSecondSet
	self
		deny: aFirstSet = aSecondSet;
		deny: aFirstSet == aSecondSet;
		deny: aSecondSet = aFirstSet;
		deny: aSecondSet == aFirstSet! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:32'!
disjointFunctions
	disjointFunctions ifNil: [
		(disjointFunctions := OrderedCollection new)
			addLast: XPathSubstringFunction;
			addLast: XPathSubstringBeforeFunction;
			addLast: XPathSubstringAfterFunction.
		disjointFunctions := disjointFunctions asArray].
	^ disjointFunctions copy.! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:15'!
emptySet
	^ self setClass new! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:26'!
functions
	functions ifNil: [
		(functions := OrderedCollection new)
			addLast: XPathTrueFunction;
			addLast: XPathFalseFunction;
			addLast: XPathNotFunction.
		functions := functions asArray].
	^ functions copy.! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:15'!
setClass
	^ XPathFunctionSet! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:39'!
setWithDisjointFunctions
	^ self setClass newFrom: self disjointFunctions! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:15'!
setWithFunctions
	^ self setClass newFrom: self functions! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/20/2015 14:38'!
should: aBlock add: aFirstObject and: aSecondObject equaling: anExpectedSum
	| firstCopy secondCopy sum |

	firstCopy := aFirstObject asFunctionSet copy.
	secondCopy := aSecondObject asFunctionSet copy.
	sum := aBlock value.
	self
		assert: sum = anExpectedSum;
		deny: sum == aFirstObject;
		deny: sum == aSecondObject;
		assert: firstCopy = aFirstObject asFunctionSet;
		assert: secondCopy = aSecondObject asFunctionSet.! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/20/2015 14:47'!
should: aBlock subtract: aFirstObject from: aSecondObject equaling: anExpectedDifference
	| firstCopy secondCopy difference |

	firstCopy := aFirstObject asFunctionSet copy.
	secondCopy := aSecondObject asFunctionSet copy.
	difference := aBlock value.
	self
		assert: difference = anExpectedDifference;
		deny: difference == aFirstObject;
		deny: difference == aSecondObject;
		assert: firstCopy = aFirstObject asFunctionSet;
		assert: secondCopy = aSecondObject asFunctionSet.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 15:19'!
testAdd
	| set |

	set := self emptySet.
	self functions do: [:each |
		self
			assert: (set add: each) == each;
			assert: (set includes: each)].
	self
		assert: set = self setWithFunctions;
		assert: (self functions allSatisfy: [:each | (set add: each) == each]);
		assert: set = self setWithFunctions.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:00'!
testAsFunctionSet
	| set function |

	set := self setClass new.
	self assert: set asFunctionSet == set.

	function := self functions anyOne.
	set := function asFunctionSet.
	self assert: set = (self setClass with: function).
	! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 16:45'!
testDo
	| enumerated |

	enumerated := OrderedCollection new.
	self emptySet do: [:each | enumerated addLast: each].
	self assert: enumerated size = 0.

	self setWithFunctions do: [:each | enumerated addLast: each].
	self
		assert: enumerated size = self functions size;
		assert: (self functions allSatisfy: [:each | enumerated includes: each]).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 16:24'!
testEquals
	| set |

	set := self emptySet.
	self
		assertSet: set equals: self emptySet;
		denySet: set equals: self setWithFunctions;
		denySet: set equals: self setWithDisjointFunctions.

	set := self setWithFunctions.
	self
		assertSet: set equals: self setWithFunctions;
		denySet: set equals: self setWithDisjointFunctions.

	set := self setWithDisjointFunctions.
	self assertSet: set equals: self setWithDisjointFunctions.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 2/10/2015 19:13'!
testFunctionNamed
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self assert: (set functionNamed: each functionName) == each].
	self
		should: [set functionNamed: self absentFunction functionName]
		raise: Error.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 2/10/2015 19:14'!
testFunctionNamedIfAbsent
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self assert:
			(set
				functionNamed: each functionName
				ifAbsent: [self absentFunction]) == each].
	self assert:
		(set
			functionNamed: self absentFunction functionName
			ifAbsent: [self absentFunction]) == self absentFunction.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 16:34'!
testIncludes
	| set |

	set := self emptySet.
	self deny: (self functions anySatisfy: [:each | set includes: each]).

	set := self setWithFunctions.
	self assert: (self functions allSatisfy: [:each | set includes: each]).
	self deny: (self disjointFunctions anySatisfy: [:each | set includes: each]).

	set := self setWithDisjointFunctions.
	self assert: (self disjointFunctions allSatisfy: [:each | set includes: each]).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 2/10/2015 19:17'!
testIncludesFunctionNamed
	| set |

	set := self emptySet.
	self functions do: [:each |
		self deny: (set includesFunctionNamed: each functionName)].

	set := self setWithFunctions.
	self functions do: [:each |
		self assert: (set includesFunctionNamed: each functionName)].
	self disjointFunctions do: [:each |
		self deny: (set includesFunctionNamed: each functionName)].

	set := self setWithDisjointFunctions.
	self disjointFunctions do: [:each |
		self assert: (set includesFunctionNamed: each functionName)].! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:58'!
testMinusFunction
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self
			should: [set - each]
			subtract: each
			from: set
			equaling: (self setClass newFrom: (self functions copyWithout: each))].

	self disjointFunctions do: [:each |
		self
			should: [set - each]
			subtract: each
			from: set
			equaling: self setWithFunctions].

	self assert:
		(self functions first - self functions second) =
			(self setClass with: self functions first).
	self assert:
		(self functions first - self functions first) =
			self emptySet.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:50'!
testMinusSet
	| firstSet secondSet |

	firstSet := self setWithFunctions.
	secondSet := self emptySet.
	self
		should: [firstSet - secondSet]
		subtract: secondSet
		from: firstSet
		equaling: self setWithFunctions.

	secondSet := self setWithDisjointFunctions.
	self
		should: [firstSet - secondSet]
		subtract: secondSet
		from: firstSet
		equaling: self setWithFunctions.
	self
		should: [firstSet - firstSet]
		subtract: firstSet
		from: firstSet
		equaling: self emptySet.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:59'!
testPlusFunction
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self
			should: [set + each]
			add: each
			and: set
			equaling: self setWithFunctions].

	set := self setWithFunctions.
	self disjointFunctions do: [:each |
		self
			should: [set + each]
			add: each
			and: set
			equaling: (self setClass newFrom: (self functions copyWith: each))].

	self assert:
		(self functions first + self functions second) =
			(self setClass
				with: self functions first
				with: self functions second).

	self assert:
		(self functions first + self functions first) =
			(self setClass with: self functions first).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:44'!
testPlusSet
	| firstSet secondSet |

	firstSet := self setWithFunctions.
	secondSet := self emptySet.
	self
		should: [firstSet + secondSet]
		add: firstSet
		and: secondSet
		equaling: self setWithFunctions.
	self
		should: [firstSet + firstSet]
		add: firstSet
		and: firstSet
		equaling: self setWithFunctions.

	secondSet := self setWithDisjointFunctions.
	self
		should: [firstSet + secondSet]
		add: firstSet
		and: secondSet
		equaling:
			(self setClass newFrom:
				self setWithDisjointFunctions, self setWithFunctions).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:00'!
testPostCopy
	| set copy |

	set := self setWithFunctions.
	copy := set copy.
	self
		assert: set = copy;
		assert: set functionsByName = copy functionsByName;
		deny: set functionsByName == copy functionsByName.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 15:27'!
testRemoveIfAbsent
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self assert:
			(set
				remove: each
				ifAbsent: [self absentFunction]) == each.
		self
			assert:
				(set
					remove: each
					ifAbsent: [self absentFunction]) == self absentFunction.
		self deny: (set includes: each).
		self
			should: [set remove: each]
			raise: Error].
	self assert: set isEmpty.! !

TestCase subclass: #XPathTest
	instanceVariableNames: 'document namespacedDocument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathTest methodsFor: 'running' stamp: 'monty 3/3/2015 12:48'!
setUp
	document :=
		(XMLDOMParser on: XMLParserTest addressBookXML)
			preservesCommentNodes: true;
			parseDocument.
	namespacedDocument := 
		(XMLDOMParser on: XMLDOMParserTest addressBookNamespacePrefixedXML)
			preservesCommentNodes: true;
			parseDocument.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/26/2015 20:44'!
testAbsoluteLocationPaths
	self
		assert: (XPath for: '/' in: document) = (XMLNodeList with: document);
		assert: (XPath for: '/*' in: document) = (XMLNodeList with: document root)! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/7/2015 10:35'!
testAddFunction
	| functionSet context |

	functionSet := XPathFunctionSet default copyWithout: XPathCeilingFunction.
	self
		should: [
			XPath
				for: 'ceiling()'
				in: nil
				context: (XPathContext functions: functionSet)]
		raise: XPathUndeclaredFunctionException.

	(context := XPathContext functions: functionSet)
		addFunction: XPathCeilingFunction.
	self
		assert: (context includesFunction: XPathCeilingFunction);
		deny: (functionSet includes: XPathCeilingFunction);
		assert: (XPath for: 'ceiling(1.9)' in: nil context: context) = 2.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:31'!
testAncestorAxis
	self assert:
		(XPath for: '//ancestor::node()' in: document) =
			(document allNodesSelect: [:each | each hasChildren]) reversed! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:31'!
testAncestorOrSelfAxis
	self assert:
		(XPath
			for: '//node() [ count( child::node() ) = 0]'
			in: document) = (document allNodesSelect: [:each | each hasChildren not]).
	self assert:
		(XPath
			for: '//node( )[count(child::node( ))=0]/ancestor-or-self::node( )'
			in: document) = document allNodes reversed! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/12/2015 20:34'!
testAsXPath
	| result src |

	result := document elements.
	src := '/addressbook'.
	self
		assert: (document xPath: src asXPath) = result;
		assert: (document xPath: (XPath for: src) asXPath) = result;
		assert: (src asXPath in: document) = result.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:31'!
testAttributeAxis
	| results |

	results :=
		XPath
			for: 'addressbook/person/address/@city'
			in: document.
	self assert: results size = 4.
	results do: [:each |
		self
			assert: each class == XMLAttribute;
			assert: each name = 'city'].
	self assert:
		(results collect: [:each | each value]) asArray =
			#('Los Angeles' 'New York' 'Los Angeles' 'New York').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 15:01'!
testAttributeTypeTest
	| results |

	results := XMLNodeList new.
	document allNodesDo: [:each |
		each isElement
			ifTrue: [results addAll: each attributeNodes]
			ifFalse: [
				each isStringNode
					ifTrue: [results addLast: each]]].
	self assert: (document xPath: '//attribute::* | //text()') = results.
	self assert:
		(document xPath: '(//attribute::* | //text())/ self :: attribute()') =
			(results select: [:each | each isAttribute]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/1/2015 15:40'!
testCommentTypeTest
	| comments |

	comments := document xPath: '//comment()'.
	self
		assert: comments size = 2;
		assert: comments = (document allNodesSelect: [:each | each isComment]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/19/2015 06:25'!
testConcatFunction
	self
		assert: (XPath for: 'concat("one", "two")' in: nil) = 'onetwo';
		assert: (XPath for: 'concat("one", "two", true())' in: nil) = 'onetwotrue'.
	self
		should: [XPath for: 'concat("one")' in: nil]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:04'!
testCountFunction
	self assert:
		(XPath for: 'count(addressbook/person)' in: document) =
			(document allElementsNamed: 'person') size.
	self assert:
		(XPath for: 'addressbook/person[count(@*) >= 4]' in: document) =
			((document allElementsNamed: 'person') copyFrom: 2 to: 3).
	self
		should: [XPath for: 'count()' in: nil]
		raise: XPathArgumentNumberException.
	self
		should: [XPath for: 'count(1)' in: nil]
		raise: XPathTypeException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:31'!
testDescendantAxis
	| path results |
	path := XPath for: '//address/@city'.
	results := path in: document.
	self assert: results size = 4 & (results size > 0).
	self assert: (results at: 1) value = 'Los Angeles'.
! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/19/2015 20:37'!
testDescendantOrSelfAxis
	self
		assert: (XPath for: '//.' in: document) = document allNodes;
		assert: (XPath for: '//./node()' in: document) = document allNodes allButFirst;
		assert: (XPath for: '//~node()' in: document) = document allNodes;
		assert: (XPath for: '// self :: node()' in: document) = document allNodes;
		assert: (XPath for: '/ descendant-or-self ::node()' in: document) = document allNodes;
		assert: (XPath for: '//*' in: document) = document allElements;
		assert:
			(XPath for: '//comment()/descendant-or-self::comment()' in: document) =
				(document allNodesSelect: [:each | each isComment]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 07:07'!
testDot
	| path results |

	path := XPath for: 'addressbook/person/address/./@city'.
	results := path in: document.
	self assert: results size = 4 & (results size > 0).
	self assert: (results at: 1) value = 'Los Angeles'.
! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:11'!
testDotDot
	| path results |
	path := XPath for: 'addressbook/person/address/../address/@city'.
	results := path in: document.
	self assert: results size = 4 & (results size > 0).
	self assert: (results at: 1) value = 'Los Angeles'.
! !

!XPathTest methodsFor: 'tests' stamp: 'PH 10/7/2003 07:15'!
testDotDotTooHigh
	| path results |
	path := XPath for: 'person/address/../../../address/@city'.
	results := path in: document.
	self assert: (results size = 0).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 17:48'!
testDoubleWildcard
	self assert:
		(XPath for: 'addressbook/*:*' in: document) =
			(document allElementsNamed: 'person').
	self assert:
		(XPath for: 'book:addressbook/*:*' in: namespacedDocument) =
			(namespacedDocument allElementsNamed: 'person').
	self assert:
		(XPath
			for: '*:*/*:*'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'two' uri: 'personNS')) =
				(namespacedDocument allElementsNamed: 'person').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 09:07'!
testElementDoubleWildcard
	self assert:
		(XPath for: 'addressbook/element(*:*)' in: document) =
			(document allElementsNamed: 'person').
	self assert:
		(XPath for: 'book:addressbook/element(*:*)' in: namespacedDocument) =
			(namespacedDocument allElementsNamed: 'person').
	self assert:
		(XPath
			for: 'element(*:*)/element(*:*)'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'two' uri: 'personNS')) =
				(namespacedDocument allElementsNamed: 'person').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 16:48'!
testElementExpandedNameTest
	| persons |

	persons := namespacedDocument allElementsNamed: 'person'.
	self assert:
		(namespacedDocument xPath:
			'element( Q{bookNS}addressbook ) / element( Q{personNS}person )') = persons.
	self assert: (namespacedDocument xPath: 'element(Q{undeclared}addressbook)') isEmpty.
	self
		should: [namespacedDocument xPath: 'element(Q{bookNS} addressbook)']
		raise: XPathSyntaxException.
	self
		should: [namespacedDocument xPath: 'element(Q {bookNS}addressbook)']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 16:44'!
testElementLocalNameWildcardTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assert: (namespacedDocument xPath: 'element(book:*)/element(ps:*)') = persons;
		assert: (namespacedDocument xPath: 'element(book:*)/element(undeclared:*)') isEmpty.
	self assert:
		(XPath
			for: 'element(one:*)/element(two:*)'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) = persons.
	self assert:
		(XPath
			for: 'element(one:*)/element(one:*)'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.
	self
		should: [
			(XPath
				for: 'element(undeclared:*)'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 16:45'!
testElementPrefixWildcardTest
	| persons |

	persons := namespacedDocument allElementsNamed: 'person'.
	self
		assert: (namespacedDocument xPath: 'element(*:addressbook)/element(*:person)') = persons;
		assert: (namespacedDocument xPath: 'element(*:addressbook)/element(*:contaxt-info)') isEmpty.
	self assert:
		(XPath
			for: 'element(one:addressbook)/element(*:person)'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'one' uri: 'bookNS')) = persons.
	self assert:
		(XPath
			for: 'element(one:addressbook)/element(*:contact-info)'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'one' uri: 'bookNS')) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 16:46'!
testElementQualfiedNameTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self assert:
		(namespacedDocument xPath:
			'element(book:addressbook)/element(ps:person)') = persons.
	self assert:
		(namespacedDocument xPath:
			'element(book:addressbook)/element(undeclared:person)') isEmpty.
	self assert:
		(XPath
			for: 'element(one:addressbook)/element(two:person)'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) = persons.
	self assert:
		(XPath
			for: 'element(one:addressbook)/element(one:person)'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.
	self
		should: [
			(XPath
				for: 'element(undeclared:addressbook)'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 14:44'!
testElementTypeTest
	self assert: (document // 'element()') = document allElements! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 14:36'!
testExpandedNameTest
	| persons |

	persons := namespacedDocument allElementsNamed: 'person'.
	self
		assert: (namespacedDocument xPath: 'Q{bookNS}addressbook/Q{personNS}person') = persons;
		assert: (namespacedDocument xPath: 'Q{undeclared}addressbook') isEmpty.
	self
		should: [namespacedDocument xPath: 'Q{bookNS} addressbook']
		raise: XPathSyntaxException.
	self
		should: [namespacedDocument xPath: 'Q {bookNS}addressbook']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 03:32'!
testFalseFunction
	self deny: 'false()' asXPath evaluate.
	self
		should: ['false(1)' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/11/2015 12:18'!
testFollowingAxis
	| following |

	following := XMLNodeList new.
	(document allElementsNamed: 'person') allButFirst do: [:person |
		person allNodesDo: [:each | following addLast: each]].
	self assert:
		(XPath
			for: 'addressbook/person/following::node()'
			in: document) = following.

	following removeAll.
	document allElementsNamed: 'person' do: [:person |
		 person nodes allButFirst do: [:node |
			node allNodesDo: [:each | following addLast: each]]].
	self assert:
		(XPath
			for: 'addressbook/person/node()/following::node()'
			in: document) = following.

	following removeAll.
	(document allElementsNamed: 'person') first descendentNodesDo: [:each |
		each previousNode
			ifNotNil: [following addLast: each]].
	(document allElementsNamed: 'person') allButFirst do: [:person |
		person allNodesDo: [:each | following addLast: each]].
	self assert:
		(XPath
			for: '//following::node()'
			in: document) = following.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/11/2015 11:26'!
testFollowingSiblingAxis
	| siblings |

	siblings := (document allElementsNamed: 'person') allButFirst.
	self assert:
		(XPath
			for: 'addressbook/person/following-sibling::person'
			in: document) = siblings.

	siblings := XMLNodeList new.
	document allElementsNamed: 'person' do: [:each |
		siblings addAll: each nodes allButFirst].
	self assert:
		(XPath
			for: 'addressbook/person/node()/following-sibling::node()'
			in: document) = siblings.

	siblings := document allNodesSelect: [:each |
		each hasParent
			and: [each previousNode notNil]].
	self assert:
		(XPath
			for: '//following-sibling::node()'
			in: document) = siblings.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 15:34'!
testFunctions
	| functionSet |

	functionSet := XPathFunctionSet default.
	self
		assert: XPathContext new functions = functionSet;
		deny: XPathContext new functions == functionSet;
		assert: (XPathContext functions: functionSet) functions == functionSet;
		assert: (XPathContext new functions: functionSet) functions == functionSet.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/8/2015 12:10'!
testIDFunction
	| xmlWithIds documentWithIds idElements |

	xmlWithIds :=
		'<!!DOCTYPE one [
			<!!ELEMENT one ANY>
			<!!ATTLIST one id ID #REQUIRED>
			<!!ELEMENT two ANY>
			<!!ATTLIST two id2 ID #IMPLIED>
			<!!ELEMENT three ANY>
			<!!ATTLIST three
				id ID #REQUIRED
				id3 CDATA #IMPLIED>
			<!!ELEMENT four ANY>
			<!!ATTLIST four id4 ID #REQUIRED>
			<!!ELEMENT five ANY>
			<!!ATTLIST five id CDATA #IMPLIED>
			<!!ELEMENT six ANY>
			<!!ATTLIST six id2 CDATA #IMPLIED>
			<!!ELEMENT seven ANY>
			<!!ATTLIST seven id4 CDATA #IMPLIED>
		]>
		<one id="first"><two/><three id="third" id3="non-id"><four id4="fourth">
			<five id="non-id">first</five>
			<six id2="non-id">third</six>
			<seven id4="non=id">non-id</seven>
		</four></three></one>'.
	documentWithIds :=
		XMLDOMParser
			parse: xmlWithIds
			usingNamespaces: true
			validation: true.

	idElements :=
		documentWithIds allElements select: [:each |
				each isNamedAny: #('one' 'three')].
	self
		assert: (documentWithIds xPath: 'id("  first  third  non-id  ")') = idElements;
		assert: (documentWithIds xPath: 'id("first third non-id")') = idElements;
		assert: (documentWithIds xPath: 'id(//*)') = idElements.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 10:06'!
testInvalidAbbreviatedLocationPath
	self
		should: ['/ /one' asXPath compile]
		raise: XPathSyntaxException.
	self
		should: ['one/ /two' asXPath compile]
		raise: XPathSyntaxException.
	self
		should: ['//' asXPath compile]
		raise: XPathSyntaxException.
	self
		should: ['//one/' asXPath compile]
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:02'!
testLangFunction
	| element |

	(element := XMLElement named: 'test')
		attributeAt: 'xml:lang' put: 'en-us'.
	self
		assert: ('lang("En")' asXPath in: element);
		assert: ('lang("en-US")' asXPath in: element);
		deny: ('lang("en-UK")' asXPath in: element);
		deny: ('lang("DE")' asXPath in: element).

	element attributeAt: 'xml:lang' put: 'En'.
	self
		assert: ('lang("En")' asXPath in: element);
		deny: ('lang("en-US")' asXPath in: element);
		deny: ('lang("en-UK")' asXPath in: element);
		deny: ('lang("DE")' asXPath in: element).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/26/2015 23:43'!
testLastFunction
	| path elements |

	path := XPath for: 'last()'.
	self assert: (path in: document) = 1.

	path := XPath for: 'last()'.
	elements := document allElementsNamed: 'person'.
	elements do: [:each | self assert: (path in: each) = elements size].

	path := XPath for: 'addressbook/person[last()]'.
	self assert:
		(path in: document) =
			(elements allButFirst: elements size - 1).
			
	path := XPath for: 'addressbook//node()[last()]'.
	self assert:
		(path in: document) =
			(XMLNodeList with: document allNodes last).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/18/2015 16:17'!
testLocalNameFunction
	self assert:
		(XPath
			for: 'local-name(addressbook/person[1]/*[last()]/preceding::*)'
			in: document) = 'contact-info'.
	self assert:
		(XPath
			for: 'local-name(addressbook/absent)'
			in: document) isEmpty.
	self assert:
		(XPath
			for: 'local-name(//comment())'
			in: document) isEmpty.
	self assert:
		(XPath
			for: 'local-name()'
			in: document root) = 'addressbook'.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 15:29'!
testLocalNameWildcardTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assert: (namespacedDocument xPath: 'book:*/ps:*') = persons;
		assert: (namespacedDocument xPath: 'book:*/undeclared:*') isEmpty.
	self assert:
		(XPath
			for: 'one:*/two:*'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) = persons.
	self assert:
		(XPath
			for: 'one:*/one:*'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.
	self
		should: [
			(XPath
				for: 'undeclared:*'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/11/2015 00:20'!
testLogicalOperators
	self
		deny: '0 or 0 or 0' asXPath evaluate;
		assert: '0 or 0 or 1' asXPath evaluate;
		deny: '1 and 1 and 0' asXPath evaluate;
		assert: '1 and 1 and 2' asXPath evaluate! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 16:02'!
testNameFunction
	| element |

	(element := XMLElement named: 'test')
		addNode: (XMLElement named: 'test-two');
		addNode: 'test-three' asXMLString.
	self
		assert: (XPath for: 'name()' in: element) = 'test';
		assert: (XPath for: 'name(//.)' in: element) = 'test'.

	element name: 'test' namespaceURI: 'defaultNS'.
	self assert: (XPath for: 'name()' in: element) = 'test'.
	self assert:
		(XPath
			for: 'name()'
			in: element
			context:
				(XPathContext new
					declarePrefix: 'one'
					uri: 'defaultNS')) = 'test'.

	element name: 'prefix:test' namespaceURI: 'prefixNS'.
	self assert: (XPath for: 'name()' in: element) = 'prefix:test'.
	self assert:
		(XPath
			for: 'name()'
			in: element
			context:
				(XPathContext new
					declarePrefix: 'two'
					uri: 'prefixNS')) = 'two:test'.

	self assert: (XPath for: 'name(//text())' in: element) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:33'!
testNamespaceAxis
	| nodes |

	nodes := XPathNodeSet new.
	namespacedDocument allElementsDo: [:each |
		nodes
			add: (XPathNamespaceNode withXMLNamespaceOnElement: each);
			add: (XPathNamespaceNode element: each name: '' uri: 'defaultNS');
			add: (XPathNamespaceNode element: each name: 'book' uri: 'bookNS');
			add: (XPathNamespaceNode element: each name: 'ps' uri: 'personNS')].
	self
		assert: nodes size = (namespacedDocument allElements size * 4);
		assert: (namespacedDocument xPath: '//namespace::*') = nodes;
		assert:
			(namespacedDocument xPath: '//namespace::book') =
				(nodes select: [:each | each name ='book']);
		assert: (namespacedDocument xPath: '//namespace::book:*') isEmpty;
		assert: (namespacedDocument xPath: '//namespace::namespace()') = nodes.

	nodes :=
		document allElementsCollect: [:each |
				XPathNamespaceNode withXMLNamespaceOnElement: each].
	self assert: (document xPath: '//namespace::*') = nodes.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/13/2015 21:32'!
testNamespaceURIFunction
	self
		assert: (document xPath: 'namespace-uri(/*)') isEmpty;
		assert: (document root xPath: 'namespace-uri()') isEmpty;
		assert: (document xPath: 'namespace-uri(/*/*)') isEmpty;
		assert: (namespacedDocument xPath: 'namespace-uri(/*)') = 'bookNS';
		assert: (namespacedDocument root xPath: 'namespace-uri()') = 'bookNS';
		assert: (namespacedDocument xPath: 'namespace-uri(/*/*)') = 'personNS'! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/13/2015 21:33'!
testNestedFunctionCalls
	self
		assert: 'number(ceiling(3.5) + floor(1.5))' asXPath evaluate = 5;
		assert: 'contains(concat(concat("one", "two"), string("three")), "two")' asXPath evaluate! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 04:13'!
testNodeSetAndNonNodeSetEquality
	self assert:
		('<list><a>1</a><a>two</a></list>' parseXML xPath:
			'list/a = 1.0
				and 1.0 = list/a
				and not(list/a = 2.0)
				and not(2.0 = list/a)
				and list/a = "two"
				and "two" = list/a
				and not(list/a = "one")
				and not("one" = list/a)
				and list/a = true()
				and true() = list/a
				and not(list/absent = true())
				and not(true() = list/absent)')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 04:13'!
testNodeSetAndNonNodeSetInequality
	self assert:
		('<list><a>1</a><b>two</b></list>' parseXML xPath:
			'list/a !!= 2.0
				and 2.0 !!= list/a
				and not(list/a !!= 1.0)
				and not(1.0 !!= list/a)
				and list/b !!= "one"
				and "one" !!= list/b
				and not(list/b !!= "two")
				and not("two" !!= list/b)
				and list/a !!= false()
				and false() !!= list/a
				and list/absent !!= true()
				and true() !!= list/absent')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/21/2015 17:22'!
testNodeSetComparing
	self assert:
		('<list>
			<a>1</a><a>2</a><a>3</a>
			<b>0</b>
			<c>4</c>
		</list>' parseXML xPath:
			'list/a < list/a
				and list/a <= list/a
				and list/a > list/a
				and list/a >= list/a
				and list/b < list/a
				and list/b <= list/a
				and not(list/b > list/a)
				and not(list/b >= list/a)
				and not(list/c < list/a)
				and not(list/c <= list/a)
				and list/c > list/a
				and list/c >= list/a')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/21/2015 17:26'!
testNodeSetEquality
	self assert:
		('<list>
			<a>one</a><a>two</a><a>three</a>
			<b>four</b>
		</list>' parseXML xPath:
			'list/a = list/a
				and list/a !!= list/a
				and not(list/b = list/a)
				and list/b !!= list/a
				and not(list/absent = list/a)
				and not(list/absent = list/absent)
				and not(list/absent !!= list/a)
				and not(list/absent !!= list/absent)')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/13/2015 05:17'!
testNormalizeSpaceFunction
	self assert:
		'normalize-space("
		    one		  
		    two
		    three
		")' asXPath evaluate = 'one two three'.
	self assert: 'normalize-space(" 	")' asXPath evaluate isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/14/2015 16:15'!
testNotFunction
	self	
		deny: 'not(true())' asXPath evaluate;
		assert: 'not(false())' asXPath evaluate;
		deny: 'not(1) or not("1")' asXPath evaluate;
		assert: 'not(0) and not("")' asXPath evaluate;
		deny: (document xPath: 'not(/node())');
		assert: (document xPath: 'not(/text())').
	self
		should: ['not()' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 20:55'!
testNumberTokenizing
	| result |

	result := '.1' asXPath evaluate.
	self assert: (result isNumber and: [result > 0]).

	result := '-.1' asXPath evaluate.
	self assert: (result isNumber and: [result < 0]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:12'!
testOneLevel
	| path results |
	path := XPath for: 'addressbook/person'.
	results := path in: document.
	self assert: (results size = 4).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/26/2015 23:10'!
testPositionFunction
	self assert:
		(XPath
			for: ' position( ) '
			in: document) = 1.

	(document allElementsNamed: 'person')
		withIndexDo: [:each :i |
			self assert: (XPath for: ' position() ' in: each) = i].

	(document allElementsNamed: 'person')
		withIndexDo: [:each :i |
			self assert:
				(XPath
					for: 'addressbook/person[position() = ', i printString, ']'
					in: document) = (XMLNodeList with: each).
			self assert:
				(XPath
					for: 'addressbook/person[ ', i printString, ' ]'
					in: document) = (XMLNodeList with: each)].! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:08'!
testPrecedence
	self
		assert: (XPath for: '1 + 2 * 3 + 4 div 2' in: nil) rounded = 9;
		assert: (XPath for: '(1 + 2) * (3 + 4) mod 5' in: nil) rounded = 1.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/12/2015 19:14'!
testPrecedingAxis
	| preceding |

	preceding := XMLNodeList new.
	(document allElementsNamed: 'person') allButLast reverseDo: [:person |
		person allNodesReverseDo: [:each | preceding addLast: each]].
	self assert:
		(XPath
			for: 'addressbook/person/preceding::node()'
			in: document) = preceding.

	preceding removeAll.
	(document allElementsNamed: 'person') reverseDo: [:person |
		 person nodes allButLast reverseDo: [:node |
			node allNodesReverseDo: [:each | preceding addLast: each]]].
	self assert:
		(XPath
			for: 'addressbook/person/node()/preceding::node()'
			in: document) = preceding.

	preceding removeAll.
	(document allElementsNamed: 'person') last
		descendentNodes allButLast reverseDo: [:each | preceding addLast: each].
	(document allElementsNamed: 'person') allButLast reverseDo: [:person |
		person allNodesReverseDo: [:each | preceding addLast: each]].
	self assert:
		(XPath
			for: '//preceding::node()'
			in: document) = preceding.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:32'!
testPrecedingSiblingAxis
	| results |

	results :=
		XPath
			for: 'addressbook/person[4]/preceding-sibling::node()'
			in: document.
	self assert: results = document root nodes allButLast reversed.

	results :=
		XPath
			for: 'addressbook/person[4]/preceding-sibling::node()/manager'
			in: document.
	self assert:
		results =
			(document root nodes allButLast
				collect: [:each | each elementAt: 'manager']).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 15:27'!
testPrefixWildcardTest
	| persons |

	persons := namespacedDocument allElementsNamed: 'person'.
	self
		assert: (namespacedDocument xPath: '*:addressbook/*:person') = persons;
		assert: (namespacedDocument xPath: '*:addressbook/*:contaxt-info') isEmpty.
	self assert:
		(XPath
			for: 'one:addressbook/*:person'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'one' uri: 'bookNS')) = persons.
	self assert:
		(XPath
			for: 'one:addressbook/*:contact-info'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'one' uri: 'bookNS')) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 04:24'!
testProcessingInstructionTypeTest
	| xml |

	xml := '<root>before<?one two?><?three four?>after</root>' parseXML.
	self assert:
		(xml xPath: '// processing-instruction ()') =
			(xml root nodes copyFrom: 2 to: 3).
	self assert:
		(xml xPath: '// processing-instruction ( "three" )') =
			(xml root nodes copyFrom: 3 to: 3).
	self assert:
		(xml xPath: '// processing-instruction ( one )') =
			(xml root nodes copyFrom: 2 to: 2).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 15:29'!
testQualfiedNameTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assert: (namespacedDocument xPath: 'book:addressbook/ps:person') = persons;
		assert: (namespacedDocument xPath: 'book:addressbook/undeclared:person') isEmpty.
	self assert:
		(XPath
			for: 'one:addressbook/two:person'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) = persons.
	self assert:
		(XPath
			for: 'one:addressbook/one:person'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.
	self
		should: [
			(XPath
				for: 'undeclared:addressbook'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/19/2015 19:31'!
testReverseForwardAxisCombination
	| nodes |

	nodes := XPathNodeSet new.
	(document allElementsNamed: 'person') allButLast do: [:person |
		person descendentNodesDo: [:each |
			nodes addLast: each]].
	self assert:
		(XPath for: '*/person[last()]/preceding::node()/node()' in: document) =
			nodes.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/26/2015 21:11'!
testSelfAxis
	self assert:
		(XPath
			for: ' / self::node() '
			in: document) = (XMLNodeList with: document).

	self assert:
		(XPath
			for: ' /*/person/self::*'
			in: document) = (document allElementsNamed: 'person').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/15/2015 22:27'!
testSorting
	| allNodes |

	allNodes := document allNodes.
	(Array
		with: (XPathNodeSet newFrom: allNodes)
		with: (XPathNodeSet newFrom: allNodes reversed))
		do: [:nodeSet |
			self
				assert: nodeSet sorted = allNodes;
				assert: nodeSet reverseSorted = allNodes reversed;
				assert: nodeSet sorted reversed sorted = allNodes;
				assert: nodeSet reverseSorted reversed reverseSorted = allNodes reversed].! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:17'!
testStartsWithFunction
	self
		assert: 'starts-with("onetwothree", "one")' asXPath evaluate;
		deny: 'starts-with("onetwothree", "ONE")' asXPath evaluate;
		assert: 'starts-with("123", 1)' asXPath evaluate! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 20:08'!
testStringLengthFunction
	self
		assert: 'string-length("test")' asXPath evaluate = 4;
		assert: 'string-length(false())' asXPath evaluate = 5;
		assert: ('string-length()' asXPath in: 'string-value' asXMLString) = 12! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 06:40'!
testSubstringAfterFunction
	self
		assert: 'substring-after("onetwothree", "one")' asXPath evaluate = 'twothree';
		assert: 'substring-after("onetwothree", ''two'')' asXPath evaluate = 'three';
		assert: 'substring-after("onetwothree", "three")' asXPath evaluate isEmpty;
		assert: 'substring-after("onetwothree", "four")' asXPath evaluate isEmpty! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 06:40'!
testSubstringBeforeFunction
	self
		assert: 'substring-before("onetwothree", "one")' asXPath evaluate isEmpty;
		assert: 'substring-before("onetwothree", ''two'')' asXPath evaluate = 'one';
		assert: 'substring-before("onetwothree", "three")' asXPath evaluate = 'onetwo';
		assert: 'substring-before("onetwothree", "four")' asXPath evaluate isEmpty! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 20:41'!
testSubstringFunction
	self
		assert: 'substring("onetwothree", 4, 3)' asXPath evaluate = 'two';
		assert: 'substring("onetwothree", 4)' asXPath evaluate = 'twothree';
		assert: 'substring("onetwothree", -3, 2)' asXPath evaluate = 'on';
		assert: 'substring("onetwothree", 12, 2)' asXPath evaluate isEmpty;
		assert: 'substring("", 0, 1)' asXPath evaluate isEmpty;
		assert: 'substring("", 0)' asXPath evaluate isEmpty! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/14/2015 14:15'!
testSumFunction
	self assert:
		('<nums><one>1</one><two>2</two><three>3</three></nums>' parseXML
			xPath: 'sum(/nums/*)') = 6.
	self
		should: ['sum()' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 15:01'!
testTextTypeTest
	self assert:
		(document xPath: ' //text( ) ') =
			(document allNodesSelect: [:each | each isStringNode])! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:12'!
testThreeLevel
	| path results |
	path := XPath for: '*/person/contact-info/email'.
	results := path in: document.
	self assert: (results size = 2).! !

!XPathTest methodsFor: 'tests' stamp: 'hfm 5/18/2013 16:58'!
testTopLevel
	| path results elem |
	path := XPath for: 'addressbook'.
	results := path in: document.
	
	self assert: results size = 1.
	
	elem := results first.
	self assert: elem isElement.
	self assert: elem name = 'addressbook'.
	self assert: elem elements size = 4.
	self assert: elem elements first isElement.
	self assert: elem elements first name = 'person'.
	self assert: elem elements last name = 'person'.	! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 06:56'!
testTranslateFunction
	self
		assert: 'translate("onetwothree", "oe", "O")' asXPath evaluate = 'OntwOthr';
		assert: 'translate("onetwothree", "oe", "OEN")' asXPath evaluate = 'OnEtwOthrEE';
		assert: 'translate("onetwothree", "z", "x")' asXPath evaluate = 'onetwothree'! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 03:32'!
testTrueFunction
	self assert: 'true()' asXPath evaluate.
	self
		should: ['true(1)' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:12'!
testTwoLevel
	| path results |
	path := XPath for: '/*/person/address'.
	results := path in: document.
	self assert: (results size = 4).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/18/2015 14:03'!
testUndeclaredFunctions
	self
		should: [XPath for: 'undeclared()/addressbook' in: document]
		raise: XPathUndeclaredFunctionException.
	self
		should: [XPath for: 'addressbook[undeclared()]' in: document]
		raise: XPathUndeclaredFunctionException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/18/2015 14:03'!
testUndeclaredVariables
	self
		should: [XPath for: '$undeclared/addressbook' in: document]
		raise: XPathUndeclaredVariableException.
	self
		should: [XPath for: 'addressbook[$undeclared]' in: document]
		raise: XPathUndeclaredVariableException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 05:21'!
testUnion
	self assert:
		(XPath
			for:
				'addressbook/person[position() = 1 or position() = 4] |
				(addressbook/person[4]/ preceding-sibling::* [position() < 4])'
			in: document) = (document allElementsNamed: 'person')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/9/2015 20:09'!
testVariables
	| path context results |

	path := XPath for: '$doc/addressbook/person[.//address[@city=$city]]'.
	(context := XPathContext new)
		variableAt: 'doc' put: document;
		variableAt: 'city' ifAbsentPut: ['Los Angeles'].
	results := path in: nil context: context.
	self
		assert: results size = 2;
		assert:
			results =
				((document allElementsNamed: 'person') select: [:each |
					((each elementAt: 'address') attributeAt: 'city') = 'Los Angeles']).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 01:38'!
testWildcard
	| path results |

	path := XPath for: 'addressbook/*'.
	results := path in: document.
	self
		assert: results size = 4;
		assert: results = (document allElementsNamed: 'person')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/9/2015 07:53'!
testWildcardDescendant
	| path results |

	path := XPath for: '//*'.
	results := path in: document.
	self
		assert: (results size = 28);
		assert: results = document descendentElements! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 01:41'!
testWildcardDescendantAttribute
	| path results |
	path := XPath for: '//*/@city'.
	results := path in: document.
	self
		assert: results size = 4;
		assert:
			results =
				((document allElementsSelect: [:each |
					each includesAttribute: 'city'])
					collect: [:each | each attributeNodeAt: 'city']).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/17/2015 04:22'!
testWildcardDescendantAttributeValue
	| path results |
	path := XPath for: ' // * [ @ city = "Los Angeles" ] '.
	results := path in: document.
	self
		assert: results size = 2;
		assert:
			results =
				(document allElementsSelect: [:each |
					(each isNamed: 'address')
						and: [(each attributeAt: 'city') = 'Los Angeles']]).
! !

!XMLCommentTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:15'!
testStringValue
	super testStringValue.

	self assert: (self nodeClass string: 'test') stringValue = 'test'.! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 22:14'!
testDefaultContextPosition
	self assert: self nodeClass new defaultContextPosition = 1! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 22:14'!
testDefaultContextSize
	self assert: self nodeClass new defaultContextSize = 1! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 11:06'!
testDocumentRoot
	| node |

	node := self nodeClass new.
	self assert: node documentRoot == node.! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:56'!
testHasExpandedName
	self deny: self nodeClass new hasExpandedName! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:02'!
testIsNamespace
	self deny: self nodeClass new isNamespace! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:00'!
testIsPrincipalNode
	self deny: self nodeClass new isPrincipalNode! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 10:56'!
testStringValue
	self assert: self nodeClass new stringValue isEmpty! !

XMLNodeTest subclass: #XPathNamespaceNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathNamespaceNodeTest methodsFor: 'instance creation' stamp: 'monty 3/26/2015 23:21'!
newElementWithNamespaceNode
	^ self newElementWithAttributes
		declarePrefix: 'prefix'
		uri: 'prefixNS'! !

!XPathNamespaceNodeTest methodsFor: 'instance creation' stamp: 'monty 3/10/2015 13:50'!
newNamespaceNode
	^ self nodeClass
		element: self newElementWithAttributes
		name: 'prefix'
		uri: 'prefixNS'! !

!XPathNamespaceNodeTest methodsFor: 'accessing' stamp: 'monty 2/13/2015 08:40'!
nodeClass
	^ XPathNamespaceNode! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/26/2015 23:24'!
testDefaultContextPosition
	| element |

	super testDefaultContextPosition.

	element := self newElementWithNamespaceNode.
	element namespaceNodes withIndexDo: [:each :i |
		self assert: each defaultContextPosition = i].! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/26/2015 23:24'!
testDefaultContextSize
	| element size |

	super testDefaultContextSize.

	element := self newElementWithNamespaceNode.
	size := element namespaceNodes size.
	element namespaceNodesDo: [:each |
		self assert: each defaultContextSize = size].! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:53'!
testEquals
	| firstNamespace secondNamespace |

	firstNamespace := self newNamespaceNode.
	self assert: firstNamespace = firstNamespace.

	secondNamespace :=
		self nodeClass
			element: firstNamespace element
			name: firstNamespace name
			uri: firstNamespace uri.
	self assert: firstNamespace = secondNamespace.

	secondNamespace :=
		self nodeClass
			element: self newElementWithAttributes
			name: firstNamespace name
			uri: firstNamespace uri.
	self deny: firstNamespace = secondNamespace.

	secondNamespace :=
		self nodeClass
			element: firstNamespace element
			name: 'newPrefix'
			uri: firstNamespace uri.
	self deny: firstNamespace = secondNamespace.

	secondNamespace :=
		self nodeClass
			element: firstNamespace element
			name: firstNamespace name
			uri: 'newURI'.
	self deny: firstNamespace = secondNamespace.! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testExpandedName
	self assert: self newNamespaceNode expandedName = 'prefix'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 12:58'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:08'!
testHasNamespaceURI
	self deny: self newNamespaceNode hasNamespaceURI! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:45'!
testHasPrefix
	self deny: self newNamespaceNode hasPrefix! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:16'!
testHasPrefixedNamespaceURI
	| namespace |

	namespace := self newNamespaceNode.
	self deny: (namespace hasPrefixedNamespaceURI: namespace uri).! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:02'!
testIsNamespace
	self assert: self nodeClass new isNamespace! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:01'!
testIsPrincipalNode
	self assert: self nodeClass new isPrincipalNode! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testLocalName
	self assert: self newNamespaceNode localName = 'prefix'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testName
	self assert: self newNamespaceNode name = 'prefix'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testNamespaceURI
	self assert: self newNamespaceNode namespaceURI isEmpty! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/26/2015 11:05'!
testPrefix
	"namespace nodes model prefixes, but cannot be prefixed themselves"
	self assert: self newNamespaceNode prefix isEmpty! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert: self newNamespaceNode stringValue = 'prefixNS'.! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:16'!
testURI
	self assert: self newNamespaceNode uri = 'prefixNS'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:28'!
testWriteXMLOn
	self assert: self newNamespaceNode printString = 'xmlns:prefix="prefixNS"'.
	self assert:
		(self nodeClass withXMLNamespaceOnElement: nil) printString =
			('xmlns:xml="', XMLNamespaceScope xmlNamespaceURI, '"').
	self assert:
		(self nodeClass
			element: nil
			name: ''
			uri: 'defaultNS') printString = 'xmlns="defaultNS"'.! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 11:24'!
testDocumentRoot
	| nodeWithNodes |

	super testDocumentRoot.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self assert: each documentRoot == nodeWithNodes].! !

!XMLStringTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass string: self markupString) stringValue =
			self markupString.! !
